#
# Author: 
# Description: 
#

import StdAudio
import sys

SAMPLING_RATE = 44100

# Print out some useful statistics about the given audio buffer
#     @param  audio the audio samples
def audioStats(audio):
    # Compute average of the sample values
    total = 0.0
    max = 0.0

    # Calculate the average level and the maximum (absolute) level 
    for i in range(0, len(audio)):
        total += audio[i]
        val = abs(audio[i])
        if (val > max):
            max = val
    avg = total / (len(audio))

    # Now find out the percent <1% max level and <5% max level
    below1 = 0
    below5 = 0
    for i in range(0, len(audio)):
        val = abs(audio[i])
        if (val < (0.05 * max)):
            below5 += 1
            if (val < (0.01 * max)):
                below1 += 1

    print("Samples            : ", len(audio))
    print("Length (s)         : ", len(audio) / SAMPLING_RATE)
    print("Average level      : ", avg)
    print("Max level          : ", max)             
    print("Below 1%% max (%%) : ", (below1 / len(audio) * 100.0))
    print("Below 5%% max (%%) : ", (below5 / len(audio) * 100.0))

# Change the volume level of all the audio in the passed in buffer
#     @param  audio  the audio samples
#     @param  gain   gain factor applied to each sample
#     @return        new buffer with samples at new gain level
def audioGain(audio, gain):
    result = []
    for i in range(0, len(audio)):
        result.append(audio[i] * gain)
    return result

# Reverse the entire buffer
#     @param   audio  the audio samples
#     @return         buffer of the same size but in reverse order
def audioReverse(audio):
    result = []
    #
    #     YOUR CODE GOES HERE
    #
    return result       

# Loop the given audio buffer a certain number of times
#     @param   audio  the audio samples
#     @param   loops  number of times to loop the samples
#     @return         buffer with the original audio repeated
#                     returns empty list if loops <= 0
def audioLoop(audio, loops):
    result = []
    #
    #     YOUR CODE GOES HERE
    #
    return result

# Merges two audio buffers together and return the resulting buffer.
# Does a simple average of the samples from each audio buffer.
# If buffers are different lengths, the result will be as long as the
# shorter buffer
#     @param   audio1  first audio buffer
#     @param   audio2  second audio buffer
#     @return          resulting audio buffer, its length will be as long as
#                      the shortest buffer sent in.
def audioMerge(audio1, audio2):
    result = []
    #
    #     YOUR CODE GOES HERE
    #
    return result

# Cut the passed in audio buffer to only the audio between a start and
# ending time.
#     @param   audio    the audio samples
#     @param   start    starting point in seconds
#     @param   end      ending point in seconds
#     @return           new clipped buffer
#                       returns empty list if end <= start
#                       returns empty list if start or end outside time range of audio
def audioCut(audio, start, end):
    result = []
    #
    #     YOUR CODE GOES HERE
    #
    return result;     

# Pad silence (numeric value 0) before and/or after an audio buffer
#     @param   audio   the audio samples
#     @params  before  seconds of silence to put before the audio
#     @params  after   seconds of silence to put after the audio
#     @return          resulting audio with the additional silence
#                      returns empty list if before or after < 0.0

    #
    #     YOUR CODE FOR audioPad GOES HERE
    #

# Echo the audio
#     @param   audio    the audio samples
#     @params  seconds  how many seconds in the past the echo is
#     @params  factor   factor, how loud the echo is compared to original
#     @return           resulting audio samples with the echo
#                       returns empty list if seconds < 0.0       

    #
    #     YOUR CODE FOR audioEcho GOES HERE
    #

# Program to test it all

# Check for a valid number of parameters, otherwise print out the
# command line options as a reminder.
if (len(sys.argv) < 3):
    print("AudioEffects <infile> <effect> [options]")
    print()
    print("Effects:")
    print("  play                                - play the file")
    print("  stats                               - print some stats about the audio")           
    print("  reverse                 <out file>  - reverse an audio file ")
    print("  gain    <factor>        <out file>  - change the volume by given factor ")
    print("  loop    <times>         <out file>  - loop audio given number of times")
    print("  merge   <infile2>       <out file>  - merge two audio files together ")
    print("  cut     <start> <end>   <out file>  - clip audio from [start, end] seconds")
    print("  pad     <start> <end>   <out file>  - add silence to start and/or end")
    print("  echo    <secs> <factor> <out file>  - add an echo to audio")

    print()
    print("Supports only 44.1 kHz mono 16-bit signed WAV audio files.")
else:
    # Get the required command line options
    inputFile  = sys.argv[1]
    effect     = sys.argv[2].lower()
    outputFile = ""

    # Obtain the samples from the input file
    audio  = StdAudio.read(inputFile)
    result = []

    print("Effect: " + effect)

    if effect == "play":
        StdAudio.playSamples(audio)
    elif effect == "stats":
            audioStats(audio)
    elif effect == "gain":
        if len(sys.argv) < 4:
            print("gain effect expecting option <factor>")
        else:
            gain = float(sys.argv[3])
            outputFile = sys.argv[4]
            result = audioGain(audio, gain)
    elif effect == "reverse":
        outputFile = sys.argv[3]
        result = audioReverse(audio)
    elif effect == "loop":
        if len(sys.argv) < 4:
            print("loop effect expecting option <times>")
        else:
            loops = int(sys.argv[3])
            outputFile = sys.argv[4];
            result = audioLoop(audio, loops)
    elif effect == "merge":
        if len(sys.argv) < 5:
            print("merge effect expecting option <infile2> <outfile>")
        else:
            inputFile2 = sys.argv[3]
            outputFile = sys.argv[4]
            audio2 = StdAudio.read(inputFile2)
            result = audioMerge(audio, audio2)
    elif effect == "cut":
        if len(sys.argv) < 6:
            print("cut effect expecting options <start> <end> <outfile>")
        else:
            start = float(sys.argv[3])
            end   = float(sys.argv[4])
            outputFile   = sys.argv[5]
            result = audioCut(audio, start, end)
    elif effect == "pad":
        if len(sys.argv) < 6:
            print("pad effect expecting options <start> <end> <outfile>")
        else:
            before = float(sys.argv[3])
            after  = float(sys.argv[4])
            outputFile    = sys.argv[5]
            # YOUR CODE TO CALL audioPad GOES HERE
    elif effect == "echo":
        if len(sys.argv) < 6:
            print("echo effect expecting options <secs> <factor> <outfile>")
        else:
            seconds = float(sys.argv[3])
            factor  = float(sys.argv[4])
            outputFile = sys.argv[5]
            # YOUR CODE TO CALL audioEcho GOES HERE
    else:
        print("Unknown effect: " + effect)
    # Write the output file if we have a result buffer and an output name
    if (result != None) and (len(outputFile) > 0):
        StdAudio.save(outputFile, result)
        print("Wrote " + str(len(result)) + " samples to " + outputFile)
